/**************************************
*                                     *
*     Playing AVI Files In OpenGL     *
*      Jeff Molofee's Lesson 35       *
*       http://nehe.gamedev.net       *
*                2001                 *
// Converted to Classic Mac by David Phillip Oster <mailto:oster@ieee.org> Aug 2001
// Converted to OS X (10.4) by David Phillip Oster <mailto:oster@ieee.org> Feb 2007
*                                     *
**************************************/

#include <GLUT/GLUT.h>											// Header File For The GLut32 Library
#include <stdlib.h>
#include "MacAVI.h"

MacAVIRef	theAVI;
GWorldPtr	gw;
//#include <windows.h>											// Header File For Windows
//#include <vfw.h>												// Header File For Video For Windows

// User Defined Variables
float		angle;												// Used For Rotation
int			next;												// Used For Animation
int			frame=0;											// Frame Counter
int			effect;												// Current Effect
bool		env=false;											// Environment Mapping (Default On)
bool		bg=true;											// Background (Default On)

long				lastframe;									// Last Frame Of The Stream
int					width;										// Video Width
int					height;										// Video Height
char				*pdata;										// Pointer To Texture Data
int					mpf = 20;									// Will Hold Rough Milliseconds Per Frame
Ptr gBuffer;
GLUquadricObj *quadratic;										// Storage For Our Quadratic Objects


static bool Initialize ()					// Any GL Init Code & User Initialiazation Goes Here
{
	MacInitAVI();
	// Start Of User Initialization
	angle		= 0.0f;											// Set Starting Angle To Zero
	glClearColor (0.0f, 0.0f, 0.0f, 0.5f);						// Black Background
	glClearDepth (1.0f);										// Depth Buffer Setup
	glDepthFunc (GL_LEQUAL);									// The Type Of Depth Testing (Less Or Equal)
	glEnable(GL_DEPTH_TEST);									// Enable Depth Testing
	glShadeModel (GL_SMOOTH);									// Select Smooth Shading
	glHint (GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);			// Set Perspective Calculations To Most Accurate

	quadratic=gluNewQuadric();									// Create A Pointer To The Quadric Object
	gluQuadricNormals(quadratic, GLU_SMOOTH);					// Create Smooth Normals 
	gluQuadricTexture(quadratic, GL_TRUE);						// Create Texture Coords 

	glEnable(GL_TEXTURE_2D);									// Enable Texture Mapping
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);	// Set Texture Max Filter
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);	// Set Texture Min Filter

	glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);		// Set The Texture Generation Mode For S To Sphere Mapping
	glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_SPHERE_MAP);		// Set The Texture Generation Mode For T To Sphere Mapping

	Rect r = {0, 0, 256, 256};	// must be power of 2.
	gBuffer = NewPtr(256*256*4);
	if(NULL == gBuffer || noErr != NewGWorld(&gw, 32, &r, NULL, NULL, 0)){
		SysBeep(1);
		exit(1);
	}
	LockPixels(GetGWorldPixMap(gw));	// GWorlds are born unlocked
	if(NULL != (theAVI = MacOpenAVI("face2.avi", gw, gBuffer)))		// Open The AVI File
	{
		lastframe = MacGetFrameCount(theAVI) - 1;
		mpf = MacGetDurationInMilli(theAVI) / lastframe;
	}


	return true;												// Return true (Initialization Successful)
}

static void Deinitialize (void)										// Any User DeInitialization Goes Here
{
	if(NULL != gw)
	{
		DisposeGWorld(gw);
		gw = NULL;
	}
	MacDisposeAVI(theAVI);											// Close The AVI File
	theAVI = NULL;
}

static void Keyboard(unsigned char key, int /*x*/, int /*y*/)
{

	switch(key){
	case 27:
		exit(0);
		break;
	case ' ':
		effect++;												// Change Effects (Increase effect)
		if (effect>3)											// Over Our Limit?
			effect=0;											// Reset Back To 0
		break;
	case 'B':
	case 'b':
		bg=!bg;													// Toggle Background Off/On
		break;
	case 'E':
	case 'e':
		env=!env;												// Toggle Environment Mapping Off/On
		break;
	}
}

#if 0
	if (g_keys->keyDown [VK_F1] == true)						// Is F1 Being Pressed?
	{
		ToggleFullscreen (g_window);							// Toggle Fullscreen Mode
	}
#endif

static void Update (int milliseconds)								// Perform Motion Updates Here
{

	angle += (float)(milliseconds) / 60.0f;						// Update angle Based On The Timer

	next+=milliseconds;											// Increase next Based On The Timer
	int oldframe = frame;
	frame = next/mpf;											// Calculate The Current Frame

	if (frame>=lastframe)										// Are We At Or Past The Last Frame?
	{
		frame=0;												// Reset The Frame Back To Zero (Start Of Video)
		next=0;													// Reset The Animation Timer (next)
	}
	if(oldframe != frame){
		// Create The Texture with the next frame
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, 256, 256, 0, GL_RGBA, GL_UNSIGNED_BYTE, MacGrabAVIFrame(theAVI, frame));
	}
	glutTimerFunc (20, Update, 20);
	glutPostRedisplay();
}

static void Draw (void)											// Draw Our Scene
{
	glClear (GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);		// Clear Screen And Depth Buffer



	if (bg)														// Is Background Visible?
	{
		glLoadIdentity();										// Reset The Modelview Matrix
		glBegin(GL_QUADS);										// Begin Drawing The Background (One Quad)
			// Front Face
			glTexCoord2f(1.0f, 1.0f); glVertex3f( 11.0f,  8.3f, -20.0f);
			glTexCoord2f(0.0f, 1.0f); glVertex3f(-11.0f,  8.3f, -20.0f);
			glTexCoord2f(0.0f, 0.0f); glVertex3f(-11.0f, -8.3f, -20.0f);
			glTexCoord2f(1.0f, 0.0f); glVertex3f( 11.0f, -8.3f, -20.0f);
		glEnd();												// Done Drawing The Background
	}

	glLoadIdentity ();											// Reset The Modelview Matrix
	glTranslatef (0.0f, 0.0f, -10.0f);							// Translate 10 Units Into The Screen

	if (env)													// Is Environment Mapping On?
	{
		glEnable(GL_TEXTURE_GEN_S);								// Enable Texture Coord Generation For S (NEW)
		glEnable(GL_TEXTURE_GEN_T);								// Enable Texture Coord Generation For T (NEW)
	}
	
	glRotatef(angle*2.3f,1.0f,0.0f,0.0f);						// Throw In Some Rotations To Move Things Around A Bit
	glRotatef(angle*1.8f,0.0f,1.0f,0.0f);						// Throw In Some Rotations To Move Things Around A Bit
	glTranslatef(0.0f,0.0f,2.0f);								// After Rotating Translate To New Position

	switch (effect)												// Which Effect?
	{
	case 0:														// Effect 0 - Cube
		glRotatef (angle*1.3f, 1.0f, 0.0f, 0.0f);				// Rotate On The X-Axis By angle
		glRotatef (angle*1.1f, 0.0f, 1.0f, 0.0f);				// Rotate On The Y-Axis By angle
		glRotatef (angle*1.2f, 0.0f, 0.0f, 1.0f);				// Rotate On The Z-Axis By angle
		glBegin(GL_QUADS);										// Begin Drawing A Cube
			// Front Face
			glNormal3f( 0.0f, 0.0f, 1.f);
			glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
			glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
			glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
			glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
			// Back Face
			glNormal3f( 0.0f, 0.0f,-1.f);
			glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
			glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
			glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
			// Top Face
			glNormal3f( 0.0f, 1.f, 0.0f);
			glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
			glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
			glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
			glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
			// Bottom Face
			glNormal3f( 0.0f,-1.f, 0.0f);
			glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
			glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
			glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
			glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
			// Right Face
			glNormal3f( 1.f, 0.0f, 0.0f);
			glTexCoord2f(1.0f, 0.0f); glVertex3f( 1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 1.0f); glVertex3f( 1.0f,  1.0f, -1.0f);
			glTexCoord2f(0.0f, 1.0f); glVertex3f( 1.0f,  1.0f,  1.0f);
			glTexCoord2f(0.0f, 0.0f); glVertex3f( 1.0f, -1.0f,  1.0f);
			// Left Face
			glNormal3f(-1.f, 0.0f, 0.0f);
			glTexCoord2f(0.0f, 0.0f); glVertex3f(-1.0f, -1.0f, -1.0f);
			glTexCoord2f(1.0f, 0.0f); glVertex3f(-1.0f, -1.0f,  1.0f);
			glTexCoord2f(1.0f, 1.0f); glVertex3f(-1.0f,  1.0f,  1.0f);
			glTexCoord2f(0.0f, 1.0f); glVertex3f(-1.0f,  1.0f, -1.0f);
		glEnd();												// Done Drawing Our Cube
		break;													// Done Effect 0

	case 1:														// Effect 1 - Sphere
		glRotatef (angle*1.3f, 1.0f, 0.0f, 0.0f);				// Rotate On The X-Axis By angle
		glRotatef (angle*1.1f, 0.0f, 1.0f, 0.0f);				// Rotate On The Y-Axis By angle
		glRotatef (angle*1.2f, 0.0f, 0.0f, 1.0f);				// Rotate On The Z-Axis By angle
		gluSphere(quadratic,1.3f,20,20);						// Draw A Sphere
		break;													// Done Drawing Sphere

	case 2:														// Effect 2 - Cylinder
		glRotatef (angle*1.3f, 1.0f, 0.0f, 0.0f);				// Rotate On The X-Axis By angle
		glRotatef (angle*1.1f, 0.0f, 1.0f, 0.0f);				// Rotate On The Y-Axis By angle
		glRotatef (angle*1.2f, 0.0f, 0.0f, 1.0f);				// Rotate On The Z-Axis By angle
		glTranslatef(0.0f,0.0f,-1.5f);							// Center The Cylinder
		gluCylinder(quadratic,1.0f,1.0f,3.0f,32,32);			// Draw A Cylinder
		break;													// Done Drawing Cylinder
	}
	
	if (env)													// Environment Mapping Enabled?
	{
		glDisable(GL_TEXTURE_GEN_S);							// Disable Texture Coord Generation For S (NEW)
		glDisable(GL_TEXTURE_GEN_T);							// Disable Texture Coord Generation For T (NEW)
	}
	
	glFlush ();													// Flush The GL Rendering Pipeline
	glutSwapBuffers();
}

static void ReSizeGLScene (int width, int height)	// Resize And Initialize The GL Window
{
	if (height == 0)		// Prevent A Divide By Zero By
	{
		height = 1;		// Making Height Equal One
	}

	glViewport (0, 0, width, height);	// Reset The Current Viewport
	glMatrixMode (GL_PROJECTION);	// Select The Projection Matrix
	glLoadIdentity ();		// Reset The Projection Matrix

	// Calculate The Aspect Ratio Of The Window
	gluPerspective (45.0f, (GLfloat) width / (GLfloat) height, 1.0f, 1000.0f);	// View Depth of 1000
	glMatrixMode (GL_MODELVIEW);	// Select The Modelview Matrix
	glLoadIdentity ();		// Reset The Modelview Matrix
}


int main(int argc, char *argv[])
{
	glutInit (&argc, argv);
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
	glutInitWindowSize (640, 480);
	glutInitWindowPosition (100, 100);
	glutCreateWindow (argv[0]);


	Initialize();
	glutDisplayFunc (Draw);
	glutReshapeFunc (ReSizeGLScene);
	glutKeyboardFunc (Keyboard);
	glutTimerFunc (20, Update, 20);
	glutMainLoop ();

	return 0;
}

