// MacAVI.cp
// by David Phillip Oster <mailto:oster@ieee.org> August 2001
#include "MacAVI.h"
#include <Quicktime/Quicktime.h>
#include <cstring>

struct MacAVI {
	Movie	m;
	SInt16	refNum;
	SInt32	trackIndex;
	UInt32	frameCount;
	Media	media;	// the media of the video track. we don't own it.
	SampleDescriptionHandle	sdh;
	ImageSequence	seq;
	Handle		h;
	TimeValue	time;
	Ptr		buffer;	// we don't own it.
};

void MacInitAVI(){
	EnterMovies();
}

void MacShutDownAVI(){
}

MacAVIRef	MacOpenAVI(const char *filename, GWorldPtr gw, Ptr buffer){
	MacAVIRef p = NULL;
	Movie mov = NULL;
	SInt16	refNum;
	SInt16	id = movieInDataForkResID;
	Str255	s;
	FSSpec	fs;
	s[0] = std::strlen(filename);
	BlockMoveData(filename, &s[1], s[0]);
	if(noErr == FSMakeFSSpec(0, 0, s, &fs) &&
		noErr == OpenMovieFile(&fs, &refNum, fsRdPerm) &&
		noErr == NewMovieFromFile(&mov, refNum, &id, NULL, newMovieActive, NULL) &&
		NULL != (p = reinterpret_cast<MacAVIRef>(NewPtrClear(sizeof(MacAVI))))){

		p->m = mov;
		p->refNum = refNum;
		long i, iMax;
		iMax = GetMovieTrackCount(p->m);
		for(i = 1; i <= iMax; ++i){
			Track t;
			Fixed widthF, heightF;
			t = GetMovieIndTrack(p->m, i);
			GetTrackDimensions(t, &widthF, &heightF);
			if(0 != widthF && 0 != heightF){
        Rect r;
				p->buffer = buffer;
				p->h = NewHandle(0);
				p->trackIndex = i;
				p->media = GetTrackMedia(t);
				p->sdh = reinterpret_cast<SampleDescriptionHandle>(NewHandle(0));
				GetMediaSampleDescription(p->media, 0, p->sdh);
				SetMovieGWorld(p->m, gw, GetGWorldDevice(gw));
				SetMovieBox(p->m, GetPortBounds(gw, &r));
				p->frameCount = MacGetFrameCount(p);
				GoToBeginningOfMovie(p->m);
				MacGrabAVIFrame(p, 0);
				break;
			}
		}
	}
	return p;
}

SInt32 MacGetFrameCount(MacAVIRef inRef){
	SInt32		frameCount = 0;
	OSType		whichMediaType = VIDEO_TYPE;
	short		flags = nextTimeMediaSample + nextTimeEdgeOK;
	TimeValue	duration;
	TimeValue	theTime = 0;
	
	while (theTime >= 0) {
		frameCount++;
		GetMovieNextInterestingTime(inRef->m,
									flags,
									1,
									&whichMediaType,
									theTime,
									0,
									&theTime,
									&duration);

		//	after the first interesting time, don't include the time we
		//  are currently at.

		flags = nextTimeMediaSample;
	} // while
	return frameCount;
}

SInt32 MacGetDurationInMilli(MacAVIRef inRef){
	TimeScale ts = GetMovieTimeScale(inRef->m);
	return (GetMovieDuration(inRef->m)*1000)/ts;
}

void MacDisposeAVI(MacAVIRef inRef){
	if(NULL == inRef){
		return;
	}
	if(0 != inRef->seq){
		CDSequenceEnd(inRef->seq);
		inRef->seq = 0;
	}
	if(NULL != inRef->h){
		DisposeHandle(inRef->h);
		inRef->h = NULL;
	}
	if(NULL != inRef->sdh){
		DisposeHandle(reinterpret_cast<Handle>(inRef->sdh));
		inRef->sdh = NULL;
	}
	if(NULL != inRef->m){
		DisposeMovie(inRef->m);
		inRef->m = NULL;
	}
	if(0 != inRef->refNum){
		CloseMovieFile(inRef->refNum);
		inRef->refNum = 0;
	}
	DisposePtr(reinterpret_cast<Ptr>(inRef));
}


// GWorlds have three problems with respect to GL Textures:
// 1.) they are upside down. 
//		(this could be fixed in the u v texture coordinate space)
// 2.) the components of each pixel are in a different order: ARGB on Mac, RGBA on openGL
//		(this could be fixed by pointing at the &(**pmh).baseAddr[1])
// 3.) there is arbitrary buffering at the end of each scan line.
//		(this could be fixed with QTNewGWorldFromPtr(), in Quicktime 4.)
// this copies from the GWorld to the buffer, fixing all three problems.
// Note: the three problems could be fixed by other means at zero cost, as above
static void GWorldToBuffer(GWorldPtr gw, Ptr buffer){
	Ptr src, dest, srcbase;
	dest = buffer;
	PixMapHandle pmh = GetGWorldPixMap(gw);
	srcbase = GetPixBaseAddr(pmh);
	SInt32 x, y;
	SInt32 rowBytes = 0x3FFF & (**pmh).rowBytes;
	SInt32 xMax = (**pmh).bounds.right - (**pmh).bounds.left;
	SInt32 yMax = (**pmh).bounds.bottom - (**pmh).bounds.top;
	srcbase += (yMax-1)*rowBytes;
	for(y = 0; y < yMax; ++y, srcbase -= rowBytes){
		src = srcbase;
		for(x = 0; x < xMax; ++x, dest += 4, src += 4){
			dest[0] = src[1];
			dest[1] = src[2];
			dest[2] = src[3];
		}
	}
}

// grab the frame and retun it in the buffer suitable for passing to glTexImage2D
Ptr MacGrabAVIFrame(MacAVIRef inRef, int frame){
	TimeValue duration;
	// get the next frame of the source movie
	short 	flags = nextTimeMediaSample;
	OSType	whichMediaType = VIDEO_TYPE;

	// if this is the first frame, include the frame we are currently on
	if(frame == 0){
		flags |= nextTimeEdgeOK;
	}
	// skip to the next interesting time and get the duration for that frame
	GetMovieNextInterestingTime(inRef->m,
								flags,
								1,
								&whichMediaType,
								inRef->time,
								0,
								&inRef->time,
								&duration);
	if(-1 == inRef->time){
		inRef->time = 0;
	}
	// set the time for the frame and give time to the movie toolbox
	SetMovieTimeValue(inRef->m, inRef->time);
	MoviesTask(inRef->m, 0);	// this grabs the next frame
	GWorldPtr 	gw;
	GDHandle	gd;
	GetMovieGWorld(inRef->m, &gw, &gd);
	GWorldToBuffer(gw, inRef->buffer);
	return inRef->buffer;
}

